<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
	
	/*
	 * [OBSOLETE] Enregistre toutes les promos d'un coup POUR 1 PRODUIT (ne pas mélanger les produits)
	 * ex. : 		
		 <productdiscounts>
                <discounts from="2018-11-23 00:00:00" to="2018-11-29 23:59:59" solde="0" type="percentage" tax="1">
                        <discount sku="50063307" ean="4054765246759">35.00</discount>
                        <discount sku="50063321" ean="4054765246773">35.00</discount>
                        <discount sku="50063338" ean="4054765246780">35.00</discount>
                        <discount sku="50063314" ean="4054765246766">35.00</discount>
                        <discount sku="50063291" ean="4054765246742">35.00</discount>
                </discounts>
                <discounts from="2018-11-30 08:00:00" to="2018-12-20 23:59:59" solde="1" type="percentage" tax="1">
                        <discount sku="50063307" ean="4054765246759">50.00</discount>
                        <discount sku="50063321" ean="4054765246773">50.00</discount>
                        <discount sku="50063338" ean="4054765246780">50.00</discount>
                        <discount sku="50063314" ean="4054765246766">50.00</discount>
                        <discount sku="50063291" ean="4054765246742">50.00</discount>
                </discounts>
        </productdiscounts>
	 */
	class PolarisLinkImport_productdiscounts extends PolarisLinkImportBase 
	{	
		protected function OnImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_PROMOS)
				return;	// Pas d'importation de promotions !
			
			// On se positionne sur le bon shop !
			$this->SetPricesContext();
			
			// Détection du produit : collection des codes			
			$id_product = null;
			$skuResolver = array();
			$discounts = array();
						
			// Parcours des réductions et résolution des sku
			foreach ($data->discounts as $discount)
			{
				$info = array(
								"from" => (string) $discount["from"],
								"to" => (string) $discount["to"],
								"solde" => $this->FirstNonEmpty((string) $discount["solde"], "0"),
								"type" => $this->FirstNonEmpty((string) $discount["type"], "amount"),
								"tax" => $this->FirstNonEmpty((string) $discount["tax"], "0"),
								"attrs" => array(),
							);	

				$info["slt"] = $info["from"].'/'.$info["to"]."/".$info["type"]."/".$info["tax"];
							
				foreach ($discount->discount as $d)
				{
					$sku = (string) $d["sku"];
					if (!empty($sku))
					{						
						$ean = (string) $d["ean"];
						
						if (!isset($skuResolver[$sku]))
						{
							$locator = $this->GetProductCombinationFor($sku, $ean, false, true, true);
							if (!empty($locator))
							{
								if (empty($id_product))
									$id_product = $locator["id_product"];
								
								if ($locator["id_product"] == $id_product)						
									$skuResolver[$sku] = (int) $locator["id_product_attribute"];
							}
						}
						
						$l = isset($skuResolver[$sku]) ? $skuResolver[$sku] : 0;
						$deleted = isset($d["deleted"]) ? (bool) $d["deleted"] : false;
						if (!empty($l) && !$deleted)
							$info["attrs"][$l] = array( "id" => NULL, "o" => 0, 
													"n" => round($info["type"] == "percentage" ? ((float) floatval((string) $d) /(float) 100.0) : (float) floatval((string) $d), 6),
													"d" => $deleted
												);
					}
				}
				
				$discounts[] = $info;
			}
			
			// Localisation du produit
			if (empty($id_product))
				return;		// produit inconnu

			// Chargement de toutes les combinaisons du produit ...
			$sql = "SELECT pa.id_product_attribute AS id FROM "._DB_PREFIX_."product_attribute pa WHERE pa.id_product=".(int)$id_product;				
			foreach (Db::getInstance()->executeS($sql, true, false) as $row)
			{
				for ($i = 0; $i < count($discounts); $i++)
				{
					if (!isset($discounts[$i]["attrs"][$row["id"]]))
						$discounts[$i]["attrs"][(int) $row["id"]] = array( "id" => NULL, "o" => 0, "n" => 0, "d" => false );
				}
			}
			
			// On parcours l'arbre pour savoir si on peut positionner une promo toute déclinaison confondue ...
			for ($i = 0; $i < count($discounts); $i++)
			{
				$reductionMinimum = -1;

				foreach ($discounts[$i]["attrs"] as $pa)
				{
					if (!$pa["d"] && $pa["n"] >= 0)
					{
						if ($reductionMinimum < 0)
							$reductionMinimum = $pa["n"];

						if ($reductionMinimum > $pa["n"])
							$reductionMinimum = $pa["n"];
					}
				}
				
				$newRules = array();
				
				if ($reductionMinimum > 0)				
					$newRules[0] = array( "id" => NULL, "o" => 0, "n" => $reductionMinimum, "d" => false );
				
				// Suppression des règles qui répète la règle principale !				
				foreach ($discounts[$i]["attrs"] as $id_attr => $rule)
				{
					if ($rule["n"] > 0 && $rule["n"] != $reductionMinimum)
						$newRules[$id_attr] = $rule;
				}

				$discounts[$i]["attrs"] = $newRules;
			}
			
			// Chargeons les règles de prix spécifiques pour ce produit
			$recycle = array();
			
			$regles = SpecificPrice::getByProductId($id_product);
			foreach ($regles as $regle_existante)
			{
				// On considère uniquement les éléments qui nous concerne
				if ($regle_existante["id_specific_price_rule"] != 0 ||
					$regle_existante["id_currency"] != 0 ||
					$regle_existante["id_country"] != 0 ||
					$regle_existante["id_group"] != 0 ||
					$regle_existante["id_customer"] != 0 ||
					$regle_existante["from_quantity"] != 1)
					continue;

				$slt = $regle_existante["from"]."/".$regle_existante["to"]."/".$regle_existante["reduction_type"]."/".$regle_existante["reduction_tax"];
				$used = false;
				
				for ($i = 0; $i < count($discounts); $i++)
				{
					$discount = $discounts[$i];
					
					if ($slt == $discount["slt"])
					{
						if (isset($discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]]))
						{
							if (!$discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]]["d"])
							{
								$used = true;
								$discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]]["id"] = $regle_existante["id_specific_price"];
								$discounts[$i]["attrs"][(int) $regle_existante["id_product_attribute"]]["o"] = $regle_existante["reduction"];
							}
						}
					}
				}
				
				if (!$used)
					$recycle[] = $regle_existante["id_specific_price"];
			}

			foreach ($discounts as $discount)
			{
				foreach ($discount["attrs"] as $id_attr => $d)
				{
					if (!$d["d"] && (empty($d["id"]) || $d["o"] != $d["n"]))
					{				
						if (empty($d["id"]))
						{
							if (count($recycle) > 0)							
								$spePrice = new SpecificPrice(array_shift($recycle));
							else
								$spePrice = new SpecificPrice();
						}
						else
							$spePrice = new SpecificPrice($d["id"]);

						$spePrice->id_product = $id_product;
						$spePrice->id_product_attribute = $id_attr;
						$spePrice->id_specific_price_rule = 0;
						$spePrice->id_shop = Shop::getContextShopID();
						$spePrice->id_shop_group = Shop::getContextShopGroupID();
						$spePrice->id_currency = 0;
						$spePrice->id_country = 0;
						$spePrice->id_group = 0;
						$spePrice->id_customer = 0;
						
						$spePrice->from = $this->GetDateTime($discount["from"]);
						$spePrice->to = $this->GetDateTime($discount["to"]);
						$spePrice->price = -1;
						$spePrice->from_quantity = 1;
						
						$spePrice->reduction = $d["n"];
						$spePrice->reduction_tax = $discount["tax"];
						$spePrice->reduction_type = $discount["type"];
						$spePrice->save();
					}					
				}
			}			
			
			// Pour tous les prix spécifiques à recycler => poubelle !
			foreach ($recycle as $id)
			{
				$sp = new SpecificPrice($id);
				$sp->delete();
			}			
		}
	}
	