<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

if (!defined('_PS_VERSION_'))
  exit;

require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_product.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_manufacturer.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_category.php");

class AdminPolarisLinkSynchroFromProdController extends ModuleAdminController
{
    private $Link;
    private $ProductionURL;

    public function __construct()
	{
		$this->context = Context::getContext();			
		$this->bootstrap = true;        
					
		parent::__construct();		
    }

    private function getCfg()
    {
        $link = null;
        foreach (PolarisLinkSite::loadConfig() as $cfg)
        {
            if (!isset($_REQUEST["id_link"]))
                $_REQUEST["id_link"] = $cfg->LinkID;    // Si aucun lien précisé, on prend le lien par défaut ...
                
            if ($cfg->LinkID == $_REQUEST["id_link"])
            {
                $link = $cfg;
                break;
            }
        }
        if (!$link)
            throw new Exception("Configuration non trouvée");

        $this->Link = $link;

        $url_production = empty($link->ProductionURL) ? "" : base64_decode($link->ProductionURL);
        if (empty($url_production))
            throw new \Exception("Impossible de trouver l'URL de production !");

        $this->ProductionURL = $url_production;

        // Vérifier que l'URL de production n'est pas la même que celle actuelle !
        $url_production = empty($cfg->ProductionURL) ? "" : base64_decode($cfg->ProductionURL);
        $url_production = substr($url_production, strpos($url_production, "//"));
        $url_current = substr(_PS_BASE_URL_.__PS_BASE_URI__, strpos(_PS_BASE_URL_.__PS_BASE_URI__, "//"));

        if ($url_production == $url_current)
            throw new \Exception("L'URL de production est identique à celle de préproduction !");
    }

    private function askProdFor($item, $sinceId)
    {
        $auth = base64_encode($this->Link->AccessKey.":");
        $context = stream_context_create([
            "http" => [
                "header" => "Authorization: Basic $auth"
            ]
        ]);

        $config = require _PS_CACHE_DIR_ . 'appParameters.php';
        $must_have = sha1("PolarisLinkMigrateChallenge".date("Y-m-d").$config['parameters']['secret']);

        $url = $this->ProductionURL . "?fc=module&module=polarislink&controller=api_migrate&item=".$item."&sinceId=".$sinceId."&secure_token=".$must_have;
        $contents = file_get_contents($url, false, $context);

        if ($contents === FALSE)
            throw new \Exception("Impossible de joindre la production : ".(error_get_last())["message"]." !");

        return json_decode($contents, true);
    }

    private function import_object(&$state, 
        &$objectInfo, 
        &$options)
    {
        try
        {
            $changed = false;
            $id = $objectInfo["id"];            

            $objectClass = $options["object"];

            $item = new $objectClass($id);
            $create_mode = $item->id != $id;

            $ignoreChanges = isset($options["ignoreFields"]) && !empty($options["ignoreFields"]) ? $options["ignoreFields"] : [];
            $ignoreChanges[] = "date_upd";
            //$ignoreChanges[] = "date_add";

            $langFields = isset($options["langFields"]) && !empty($options["langFields"]) ? $options["langFields"] : [];

            if (isset($options["beforeImport"]))
                $options["beforeImport"]($objectInfo);

            $changes = [];
            foreach ($objectInfo as $k => $v)
            {
                if (property_exists($item, $k))
                {
                    if ($create_mode || !in_array($k, $ignoreChanges))
                    {
                        if (in_array($k, $langFields))
                        {
                            foreach ($v as $lang_id => $lang_txt)
                            {
                                if ($item->$k[$lang_id] != $lang_txt)
                                {
                                    $changed = true; 
                                    $item->$k[$lang_id] = $lang_txt;                                
                                    $changes[$k] = $k;
                                }
                            }
                        }
                        else if ($item->$k != $v)
                        {
                            $changed = true; 
                            $item->$k = $v; 
                            $changes[$k] = $k;
                        }
                    }
                }
            }
            
            if ($create_mode)
            {
                $state["msg"][] = sprintf("Importation de %s #%d", $objectClass, $id);                
                $item->id = $id;
                $item->force_id = true;
                $item->add();
            }
            else if ($changed)
            {   
                $state["msg"][] = sprintf("Importation modifications %s (%s) #%d", $objectClass, implode(', ', $changes), $id);
                $item->update();                
            }
            
            if ($create_mode || $changed)
            {
                if (isset($options["afterImport"]))
                    $options["afterImport"]($objectInfo);

                return true;
            }

            return false;
        }
        catch (\Exception $ex)
        {            
            $state["errors"]++;
            $state["msg"][] = sprintf("Erreur à l'importation %s #%d : ", $objectClass, $id) . $ex->getMessage();
        }

        return false;
    }

    /**
     * For a given product_attribute reference, returns the corresponding id.
     *
     * @param int $idProduct
     * @param string $reference
     *
     * @return int id
     */
    public static function searchPAByReference($reference)
    {
        if (empty($reference)) {
            return 0;
        }

        $query = new DbQuery();
        $query->select('pa.id_product_attribute');
        $query->from('product_attribute', 'pa');
        $query->where('pa.reference = \'' . pSQL($reference) . '\'');

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($query);
    }

    public static function __replaceProductAttribute(&$detailInfo)
    {
        $product = new Combination((int) $detailInfo["product_attribute_id"]);        
        if (empty($product->id_product) || $product->reference != $detailInfo["reference"])
        {
            // Recherche par référence
            $pa_id = self::searchPAByReference($detailInfo["reference"]);
            if ($pa_id != 0)
                $product = new Combination($detailInfo["product_attribute_id"]);
        }
        
        if (empty($product->id))
        {
            $detailInfo["product_id"] = NULL;
            $detailInfo["product_attribute_id"] = NULL;
        }
        else
        {
            $detailInfo["product_id"] = (string) empty($product->id_product) ? 0 : $product->id_product;
            $detailInfo["product_attribute_id"] = (string) empty($product->id) ? 0 : $product->id;
        }
    }

    public static function __recomputeTaxes(&$detailInfo)
    {
        //self::$__recomputeTaxesOrders[] = $detailInfo["id"];

        $detail = new OrderDetail($detailInfo["id"]);
        $detail->updateTaxAmount(new Order($detailInfo["id_order"]));
    }

    public function ajaxProcessImport()
    {
        try
        {    
            $this->getCfg();
            $db = Db::getInstance();            

            $stepsInfo = [            
                /*"products" => [
                    "object" => "Product",
                    "langFields" => ["description", "description_short", "link_rewrite", "meta_description", "meta_keywords", "meta_title", "name", "available_later", "available_now"],
                ],

                "products_attributes" => [
                    "object" => "Combination",                    
                ],

                "products_suppliers" => [
                    "object" => "ProductSupplier",
                ],*/

                "customers" => [
                        "object" => "Customer",
                    ],                 

                "addresses" => [
                        "object" => "Address",
                    ],  
                "vouchers" => [
                        "object" => "CartRule",
                        "langFields" => ["name"],
                    ], 
                "messages" => [
                        "object" => "Message",
                    ],  
                "customerThreads" => [
                    "object" => "CustomerThread",
                    ],  
                "customerMessages" => [
                    "object" => "CustomerMessage",
                    ],

                "orders" => [
                        "object" => "Order",                        
                    ],                     
                "orderDetails" => [
                        "object" => "OrderDetail",
                        "beforeImport" => "AdminPolarisLinkSynchroFromProdController::__replaceProductAttribute",
                        "afterImport" => "AdminPolarisLinkSynchroFromProdController::__recomputeTaxes",
                    ],                    
                "orderInvoice" => [
                    "object" => "OrderInvoice",
                ],

                "orderHistory" => [
                    "object" => "OrderHistory",
                ],

                "orderReturn" => [
                    "object" => "OrderReturn",
                ],

                "orderSlip" => [
                    "object" => "OrderSlip",
                ],                

                "orderPayment" => [
                    "object" => "OrderPayment",
                ],
            ];

            $step = isset($_REQUEST["step"]) && !empty($_REQUEST["step"]) ? $_REQUEST["step"] : null;            
            $id = isset($_REQUEST["id"]) ? $_REQUEST["id"] : 0;

            if ($step === null)
            {
                foreach ($stepsInfo as $k => $v)
                {
                    $step = $k;
                    break;
                }
            }

            // Vérifier si step existe ??
            if (!isset($stepsInfo[$step]))
                throw new \Exception("Etape d'importation inconnue !");

            $result = [
                "status" => "working", 
                "msg" => [],
                "errors" => 0,
                "step" => $step,
                "id" => $id,
            ];
            
            $items = $this->askProdFor($step, $id);
            if (count($items) == 0)
            {
                $result["id"] = 0;
                $result["step"] = "";
                

                $ok = false;
                foreach ($stepsInfo as $k => $v)
                {
                    if ($ok)
                    {
                        // trouvé !
                        $result["step"] = $k;
                        break;
                    } else if ($k == $step)
                        $ok = true;
                }

                if (!$ok)
                    throw new \Exception("Erreur dans la sélection de l'étape suivante");

                // fini !
                if (empty($result["step"]))
                    $result["status"] = "done";                
            }
            else
            {
                if (method_exists($this, "import_".$step))
                    $method = "import_".$step;
                else
                    $method = "import_object";

                foreach ($items as $item)
                {
                    $this->$method($result, $item, $stepsInfo[$step]);
                    $result["id"] = $item["id"];
                }
            }

            header("Content-Type: application/json");
            echo Tools::JsonEncode($result);
        }
        catch (Exception $ex)
        {
            header("Content-Type: application/json");
            echo Tools::JsonEncode(["status" => "error", "error"=> $ex->getMessage()]);
        }

        return;
    }
    
	/* On procède à l'affichage */
	public function renderList()
	{		
        try
        {            
            $this->getCfg();
    
            $output = '';            
            $output .= '<ul id="pl-prodsynchro"></ul>';
            $output .= '<p id="pl-prodsynchro-status">Synchronisation <span id="pl-prodsynchro-step">...</span> </p>';
            $output .= '<script src="'.__PS_BASE_URI__.'modules/polarislink/views/prodsync.js"></script>';            
        

            return $output;
        }
        catch (Exception $ex)
        {                
            return '<div class="alert alert-danger">'.Tools::displayError($ex->getMessage()).'</div>';
        }            
	}
}